#ifndef _UNISTD_H
#define _UNISTD_H

/* NOTE:
 * in prototypes used here, do *not* use things
 * like size_t, uid_t, etc., since these
 * may not be defined. Use the macros
 * from <compiler.h> instead.
 */
#ifndef _COMPILER_H
#include <compiler.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif

/* see Posix standard -- 1003.1 sect 2.10 */

#ifndef NULL
#define NULL __NULL
#endif

#ifndef F_OK
/* file access stuff */
#define	F_OK			0
#define	X_OK			1
#define	W_OK			2
#define	R_OK			4
#endif

#ifndef SEEK_SET
/* lseek() origins */
#define	SEEK_SET	0		/* from beginning of file */
#define	SEEK_CUR	1		/* from current location */
#define	SEEK_END	2		/* from end of file */
#endif

#define _POSIX_JOB_CONTROL
#if 0
#define _POSIX_VERSION		0L		/* <- NON-CONFORMING */
#endif
#define _POSIX_CHOWN_RESTRICTED 0
#ifndef _TERMIOS_H
#define _POSIX_VDISABLE		0
#endif

#if 0					/* this depends on the file system */
#define _POSIX_NO_TRUNC		-1
#endif

/* 1003.1 section 8.2.1.2 */
#define STDIN_FILENO	0
#define STDOUT_FILENO	1
#define STDERR_FILENO	2

/* sysconf() values */
#define _SC_LAST	-1	/* max limit # for sysconf - NOT POSIX */
#define _SC_MEMR_MAX	0	/* memory regions per process - NOT POSIX */
#define _SC_ARG_MAX	1	/* max length of cmdln - NOT quite POSIX */
#define _SC_OPEN_MAX	2	/* max # of open files per process - POSIX */
#define _SC_NGROUPS_MAX	3	/* max # supp gids - POSIX */
#define _SC_CHILD_MAX	4	/* max # processes/user - NOT quite POSIX */

/* pathconf() values */
#define _PC_LAST	-1	/* max limit # for pathconf - POSIX (!?) */
#define _PC_IOPEN_MAX	0	/* internal limit on open files - NOT POSIX */
#define _PC_LINK_MAX	1	/* max # of links - POSIX */
#define _PC_PATH_MAX	2	/* max len of a full pathname - POSIX */
#define _PC_NAME_MAX	3	/* max len of individual name - POSIX */
#define _PC_PIPE_BUF	4	/* bytes written atomically to fifo - POSIX */
#define _PC_NO_TRUNC	5	/* filename truncation - POSIX */
#ifdef __MINT__
#  define _MINT_PC_NAME_CASE 6  /* case-insensitive file-names */
#endif

#ifdef __MINT__
# if defined (__USE_MISC) && !defined (F_LOCK)
/* NOTE: These declarations also appear in <fcntl.h>; be sure to keep both
   files consistent.  Some systems have them there and some here, and some
   software depends on the macros being defined without including both.  */

/* `lockf' is a simpler interface to the locking facilities of `fcntl'.
   LEN is always relative to the current file position.
   The CMD argument is one of the following.  */

/* flock() commands */
#  define F_ULOCK	0	/* unlock */
#  define F_LOCK	1	/* lock */
#  define F_TLOCK	2	/* test and lock (non-blocking) */
#  define F_TEST	3	/* test */
__EXTERN int		lockf	__PROTO((int, int, long));
# endif
#endif /* __MINT__ */

/* If the license of your software does not meet the requirements of 
   GNU General Public License you cannot use the more sophisticated GNU
   versions of the getopt functions.  Define _AVOID_GPL in this case.
   Then your program will get linked against the original MiNTLib
   versions.  */
#ifdef _AVOID_GPL
# define getopt __getopt_old
# define optind __optind_old
# define opterr __opterr_old
# define optopt __optopt_old
# define optarg __optarg_old
#endif

__EXTERN __EXITING _exit	__PROTO((int)) __NORETURN;
__EXTERN int	access	__PROTO((const char *, int));
__EXTERN int	__access	__PROTO((const char *, int));
__EXTERN unsigned alarm	__PROTO((unsigned));
__EXTERN int	chdir	__PROTO((const char *));
__EXTERN int	chown	__PROTO((const char *, int, int));
__EXTERN int	close	__PROTO((int));
__EXTERN int	__close	__PROTO((int));
__EXTERN int	dup	__PROTO((int));
__EXTERN int	dup2	__PROTO((int, int));
__EXTERN int	__dup2	__PROTO((int, int));
__EXTERN int	execl	__PROTO((const char *, ...));
__EXTERN int	execle	__PROTO((const char *, ...));
__EXTERN int	execlp	__PROTO((const char *, ...));
__EXTERN int	execv	__PROTO((const char *, char *const *));
__EXTERN int	execve	__PROTO((const char *, char *const *, char *const *));
__EXTERN int	execvp	__PROTO((const char *, char *const *));
__EXTERN int	fork	__PROTO((void));
__EXTERN int	__fork	__PROTO((void));
__EXTERN char *	getcwd	__PROTO((char *, int));
__EXTERN _GID_T	getegid	__PROTO((void));
__EXTERN _UID_T	geteuid	__PROTO((void));
__EXTERN _GID_T	getgid	__PROTO((void));
__EXTERN int	getgroups __PROTO((int, _GID_T *));
__EXTERN char *	getlogin __PROTO((void));
__EXTERN int	getpid	__PROTO((void));
__EXTERN int	getppid	__PROTO((void));
__EXTERN int	__getppid	__PROTO((void));
__EXTERN _UID_T	getuid	__PROTO((void));
__EXTERN int	isatty	__PROTO((int));
__EXTERN int	link	__PROTO((const char *oldname, const char *newname));
__EXTERN int	__link	__PROTO((const char *oldname, const char *newname));
__EXTERN long	lseek	__PROTO((int, long, int));
__EXTERN long	__lseek	__PROTO((int, long, int));
__EXTERN int	pause	__PROTO((void));
__EXTERN int	__pause	__PROTO((void));
__EXTERN int	pipe	__PROTO((int *));
__EXTERN long	pathconf __PROTO((const char *, int));
__EXTERN int	rmdir	__PROTO((const char *));
#ifdef __MSHORT__
__EXTERN long	read	__PROTO((int, void *, __SIZE_TYPEDEF__));
__EXTERN long	__read	__PROTO((int, void *, __SIZE_TYPEDEF__));
#else
__EXTERN int	read	__PROTO((int, void *, __SIZE_TYPEDEF__));
__EXTERN int	__read	__PROTO((int, void *, __SIZE_TYPEDEF__));
#endif
__EXTERN int	setgid	__PROTO((int));
__EXTERN int	__setgid	__PROTO((int));
__EXTERN int	setuid	__PROTO((int));
__EXTERN int	__setuid	__PROTO((int));
__EXTERN unsigned sleep	__PROTO((unsigned int));
__EXTERN long	sysconf	__PROTO((int));
__EXTERN char *	ttyname	__PROTO((int));
__EXTERN int	unlink	__PROTO((const char *));
__EXTERN int	__unlink	__PROTO((const char *));
#ifdef __MSHORT__
__EXTERN long	write	__PROTO((int, const void *, __SIZE_TYPEDEF__));
__EXTERN long	__write	__PROTO((int, const void *, __SIZE_TYPEDEF__));
#else
__EXTERN int	write	__PROTO((int, const void *, __SIZE_TYPEDEF__));
__EXTERN int	__write	__PROTO((int, const void *, __SIZE_TYPEDEF__));
#endif

__EXTERN int	__bsd_getpgrp __PROTO((int));
__EXTERN int	__bsd_setpgrp __PROTO((int, int));

#ifdef __USE_BSD
__EXTERN int	gethostname __PROTO((char *__buf, __SIZE_TYPEDEF__ __len));
__EXTERN int    sethostname __PROTO((const char *__buf, __SIZE_TYPEDEF__ __len));
__EXTERN int	getpagesize __PROTO((void));
__EXTERN int    getdtablesize __PROTO ((void));
__EXTERN char *	getpass	__PROTO((const char *__prompt));
#endif

__EXTERN int	getloadavg __PROTO((double *, int));  /* Should it be here?  */

__EXTERN char *	getwd	__PROTO((char *));
__EXTERN int	getopt	__PROTO((int, char * const *, const char *));
__EXTERN char *	initstate __PROTO((unsigned int __seed, char *__arg_state, 
                                   int __n));

__EXTERN void	psignal	__PROTO((int, const char *));
__EXTERN long 	random	__PROTO((void));
__EXTERN long	_read	__PROTO((int, void *, unsigned long));
__EXTERN int	readlink __PROTO((char *__filename, char *__linkto, int __siz));
__EXTERN int	__readlink __PROTO((char *__filename, char *__linkto, int __siz));
__EXTERN void *	sbrk	__PROTO((__SIZE_TYPEDEF__));
__EXTERN void *	_sbrk	__PROTO((long));
__EXTERN int	setegid	__PROTO((int));
__EXTERN int	seteuid	__PROTO((int));
__EXTERN int	setregid __PROTO((int, int));
__EXTERN int	setreuid __PROTO((int, int));
__EXTERN char *	setstate __PROTO((char *arg_state));
__EXTERN void	srandom	__PROTO((unsigned int seed));

__EXTERN int	symlink __PROTO((const char *oldname, const char *newname));
__EXTERN int	__symlink __PROTO((const char *oldname, const char *newname));

#ifdef __MSHORT__
/* Hm, syscall() must return int, but some of the functions it calls
   actually return long or even unsigned long.  What should we do?
   Using -mshort is not portable and so we declare syscall() in a
   non-portable way here.  */
__EXTERN long	syscall	__PROTO((int, ...));
#else
__EXTERN int	syscall __PROTO((int, ...));
#endif

__EXTERN long	tell	__PROTO((int));
__EXTERN void	usleep	__PROTO((unsigned long));
__EXTERN int	vfork	__PROTO((void));
__EXTERN long	_write	__PROTO((int, const void *, unsigned long));

#ifdef __SOZOBON__
#define lsbrk _sbrk	/* backwards compatibility, don't use */
#endif

#ifndef __FAVOR_BSD
__EXTERN _PID_T getpgrp __PROTO ((void));
__EXTERN int	setpgrp	__PROTO ((void));
#else /* Favor BSD.  */
# define getpgrp __bsd_getpgrp
# define setpgrp __bsd_setpgrp
#endif

__EXTERN int	setpgid __PROTO((_PID_T, _PID_T));
__EXTERN int	setsid __PROTO((void));
__EXTERN int	__setsid __PROTO((void));
__EXTERN _PID_T	tcgetpgrp	__PROTO((int fd));
__EXTERN int	tcsetpgrp	__PROTO((int fd, _PID_T pgrp));

extern int optind;
extern int opterr;
extern char* optarg;
extern int optopt;

#if defined (__USE_BSD) || defined (__USE_XOPEN_EXTENDED)
/* Change the owner and group of the file that FD is open on.  */
__EXTERN int fchown __PROTO ((int __fd, _UID_T __owner, _GID_T __group));

#if 0
/* Change owner and group of FILE, if it is a symbolic link link the
   ownership of the symbolic link ischanged.  */
__EXTERN int lchown __PROTO ((const char* __file, _UID_T __owner,
                              _GID_T __group));
#endif
#endif

#ifdef __USE_GNU

/* Evaluate EXPRESSION, and repeat as long as it returns -1 with `errno'
   set to EINTR.  Because EINTR is fake with MiNT this is a stubs macro
   but we provide to allow writing portable code.  */

#define TEMP_FAILURE_RETRY(expression) expression

/* This variable is set nonzero at startup if the process's effective IDs
   differ from its real IDs, or it is otherwise indicated that extra
   security should be used.  */
extern int __libc_enable_secure;

#endif

#if defined(__cplusplus)
}
#endif

#endif /* _UNISTD_H */
