/* mkdir: make a new directory
 * written by Eric R. Smith and placed in the public domain
 * modified by Alan Hourihane, to check for directory and return EEXIST.
 */

#include <errno.h>
#include <limits.h>
#include <osbind.h>
#include <mintbind.h>

#ifdef __TURBOC__
# include <sys\types.h>
# include <sys\stat.h>
#else
# include <sys/types.h>
# include <sys/stat.h>
#endif

#include <unistd.h>
#include "lib.h"

extern int errno;

int 
__mkdir(_path, mode)
	const char *_path;
	mode_t mode;
{
	struct stat statbuf;
	int rv, umask;
	char pathbuf[PATH_MAX];
	char* path = (char*) _path;
	
	if (!__libc_unix_names)
	  {
	    path = pathbuf;
	    _unx2dos(_path, path, sizeof (pathbuf));
	  }

	rv = stat(path, &statbuf);	/* Stat directory */
	if (rv == 0) {			/* Does it exist ? */
		errno = EEXIST;		/* Yes, so tell user. */
		return -1;
	}

	if (errno != ENOENT) {		/* Return stat error, if other than */
					/* File not found. */
		if ((errno == EPATH) && (_enoent(path)))
			errno = ENOENT;
		return -1;
	}

	rv = Dcreate(path);
	if (rv < 0) {
		errno = -rv;
		return -1;
	}
	
	if ((umask = Pumask (0)) != -EINVAL) {
		(void) Pumask (umask);
		(void)Fchmod(path, mode & ~umask);
	}
	return 0;
}

#ifdef HAVE_WEAK_SYMBOLS
weak_alias (__mkdir, mkdir)
#endif
